@@ -1,5 +1,5 @@
 '''An object relational mapper library for Redis remote data structures.'''
-VERSION = (0, 5, 5)
+VERSION = (0, 6, 0)
  
 def get_version():
     return '.'.join(map(str,VERSION))
@@ -12,9 +12,11 @@ __homepage__ = "https://github.com/lsbardel/python-stdnet"
 CLASSIFIERS = [
                'Development Status :: 4 - Beta',
                'Environment :: Plugins',
+               'Environment :: Web Environment',
                'Intended Audience :: Developers',
                'License :: OSI Approved :: BSD License',
                'Operating System :: OS Independent',
+               'Programming Language :: Python',
                'Programming Language :: Python :: 2.6',
                'Programming Language :: Python :: 2.7',
                'Programming Language :: Python :: 3.1',
