@@ -1,28 +1,12 @@
+import json
+from hashlib import sha1
+
 from stdnet.exceptions import *
-from stdnet.utils import pickle, iteritems
+from stdnet.utils import zip, pickle, iteritems, BytesIO, encoders
 
 from .structures import Structure
 
-class default_pickler:
-    
-    @classmethod
-    def loads(cls,x):
-        return pickle.loads(x)
-    
-    @classmethod
-    def dumps(cls, x):
-        return pickle.dumps(x,2)
-
-
-class NoPickle(object):
-    
-    def loads(self, s):
-        return s
-    
-    def dumps(self, obj):
-        return obj
-
-nopickle = NoPickle()
+__all__ = ['BackendDataServer', 'BeckendQuery']
 
 
 class Keys(object):
@@ -34,33 +18,75 @@ class Keys(object):
         
     def add(self, value):
         self.value = value
+        
+        
+class BeckendQuery(object):
+    query_set = None
+    
+    def __init__(self, qs, fargs = None, eargs = None, timeout = 0,
+                 queries = None):
+        self.qs = qs
+        self.expire = max(timeout,30)
+        self.timeout = timeout
+        self.server = qs._meta.cursor
+        self.meta = qs._meta
+        self._sha = BytesIO()
+        self.build(fargs, eargs, queries)
+        code = self._sha.getvalue()
+        self._sha = code if not code else sha1(code).hexdigest()
+        self.execute()
+
+    @property
+    def sha(self):
+        return self._sha
+    
+    def __len__(self):
+        return self.count()
+
+    def has(self, val):
+        raise NotImplementedError
+    
+    def count(self):
+        raise NotImplementedError
+    
+    def build(self, fargs, eargs, queries):
+        raise NotImplementedError
+    
+    def execute(self):
+        raise NotImplementedError
+    
+    def items(self, slic):
+        raise NotImplementedError
     
 
 class BackendDataServer(object):
-    '''Generic interface for a backend database:
+    '''\
+Generic interface for a backend database:
     
-    * *name* name of database, such as **redis**, **couchdb**, etc..
-    * *params* dictionary of configuration parameters
-    * *pickler* calss for serializing and unserializing data. It must implement the *loads* and *dumps* methods.
-    '''
+:parameter name: name of database, such as **redis**, **couchdb**, etc..
+:parameter params: dictionary of configuration parameters
+:parameter pickler: calss for serializing and unserializing data.
+It must implement the *loads* and *dumps* methods.'''
+    Transaction = None
+    Query = None
     structure_module = None
-    def __init__(self, name, params, pickler = None):
+    
+    def __init__(self, name, pickler = None, charset = 'utf-8', **params):
         self.__name = name
-        timeout = params.get('timeout', 0)
-        try:
-            timeout = int(timeout)
-        except (ValueError, TypeError):
-            timeout = 0
-        self.default_timeout = timeout
         self._cachepipe = {}
-        self._keys      = {}
-        self.params     = params
-        self.pickler    = pickler or default_pickler
+        self._keys = {}
+        self.charset = charset
+        self.pickler = pickler or encoders.NoEncoder()
+        self.params = params
 
     @property
     def name(self):
         return self.__name
     
+    def disconnect(self):
+        '''Disconnect the connection.'''
+        pass
+    
     def __repr__(self):
         return '%s backend' % self.__name
     
@@ -78,70 +104,82 @@ class BackendDataServer(object):
         return len(keys)
     
     def instance_keys(self, obj):
+        '''Return a list of database keys used by instance *obj*'''
         raise NotImplementedError
     
-    def delete(self, *key):
-        "Delete one or more keys specified by ``keys``"
-        raise NotImplementedError
-    
-    def _get_pipe(self, id, typ, timeout):
-        cache  = self._cachepipe
-        cvalue = cache.get(id,None)
-        if cvalue is None:
-            cvalue = typ(timeout)
-            cache[id] = cvalue
-        return cvalue
-            
-    def commit(self):
-        '''Commit cache objects to database.'''
-        cache = self._cachepipe
-        keys = self._keys
-        # flush cache
-        self._cachepipe = {}
-        self._keys = {}
-        # commit
-        for id,pipe in iteritems(cache):
-            el = getattr(self,pipe.method)(id, pipeline = pipe)
-            el.save()
-        if keys: 
-            self._set_keys(keys)
-            
-    def get_object(self, meta, name, value):
-        '''Retrive an object from the database. If object is not available, it raises
-an :class:`stdnet.exceptions.ObjectNotFound` exception.
+    def transaction(self, pipelined = True, cachepipes = None):
+        '''Return a transaction instance'''
+        return self.Transaction(self,pipelined,cachepipes)
 
-    * *meta* :ref:`database metaclass <database-metaclass>` or model
-    * *name* name of field (must be unique)
-    * *value* value of field to search.'''
-        raise NotImplementedError
-    
-    def save_object(self, obj, commit):
-        '''Save an instance of a model to the back-end database:
+    def save_object(self, obj, transaction = None):
+        '''\
+Save or updated an instance of a model to the back-end database:
         
-        * *obj* instance of :ref:`StdModel <model-model>` to add to database
-        * *commit* If True, *obj* is saved to database, otherwise it remains in local cache.
-        '''
-        raise NotImplementedError
-    
-    def save_object(self, obj, commit = True):
-        '''Save a model object to the database:
+:parameter obj: instance of :ref:`StdModel <model-model>`
+                to add/update to the database
+:parameer transaction: optional transaction instance.'''
+        commit = False
+        if not transaction:
+            commit = True
+            transaction = self.transaction(cachepipes = obj._cachepipes)
+            
+        # Save the object in the back-end
+        if not obj.is_valid():
+            raise FieldError(json.dumps(obj.errors))
         
-        * *obj* instance of :ref:`StdModel <model-model>` to add to database
-        * *commit* If True, *obj* is saved to database, otherwise it remains in local cache.
-        '''
-        raise NotImplementedError
-    
-    def delete_object(self, obj, deleted = None, multi_field = True):
+        # We are updating the object,
+        # therefore we need to clean up indexes first
+        if obj.id:
+            try:
+                pobj = obj.__class__.objects.get(id = obj.id)
+            except obj.DoesNotExist:
+                pass
+            else:
+                self._remove_indexes(pobj, transaction)
+        else:
+            obj.id = obj._meta.pk.serialize(obj.id)
+            
+        obj = self._save_object(obj, transaction)
+        
+        if commit:
+            transaction.commit()
+        
+        return obj
+        
+    def delete_object(self, obj, transaction = None):
         '''Delete an object from the data server and clean up indices.
 Called to clear a model instance.
 :parameter obj: instance of :class:`stdnet.orm.StdModel`
-:parameter deleted: a list or ``None``. If a list, deleted keys will be appended to it.
-:parameter multi_field: if ``True`` the multifield ids (if any) will be removed. Default ``True``.
+:parameter deleted: a list or ``None``. If a list, deleted keys
+                    will be appended to it.
+:parameter multi_field: if ``True`` the multifield ids (if any)
+                        will be removed. Default ``True``.
         '''
-        raise NotImplementedError
+        commit = False
+        if not transaction:
+            commit = True
+            transaction = self.transaction()
+        
+
+        self._remove_indexes(obj, transaction)
+        self._delete_object(obj, transaction)
+        
+        if commit:
+            transaction.commit()
+            
+        return 1
+    
+    def make_objects(self, meta, ids, data):
+        make_object = meta.maker
+        for id,fields in zip(ids,data):
+            obj = make_object()
+            fields = dict(fields)
+            #fields = dict(((k.decode(),v) for (k,v) in fields))
+            obj.__setstate__((id,fields))
+            yield obj
         
     def set(self, id, value, timeout = None):
-        timeout = timeout if timeout is not None else self.default_timeout
+        timeout = timeout or 0
         value = self.pickler.dumps(value)
         return self._set(id,value,timeout)
     
@@ -185,10 +223,8 @@ Called to clear a model instance.
         return new_value
 
     def decr(self, key, delta=1):
-        """
-        Subtract delta from value in the cache. If the key does not exist, raise
-        a ValueError exception.
-        """
+        """Subtract delta from value in the cache.
+If the key does not exist, raise a ValueError exception."""
         return self.incr(key, -delta)
 
     def __contains__(self, key):
@@ -209,11 +245,20 @@ Called to clear a model instance.
         for key in keys:
             self.delete(key)
 
+    # PURE VIRTUAL METHODS
+    
     def clear(self):
         """Remove *all* values from the database at once."""
         raise NotImplementedError
-
-    # VIRTUAL METHODS
+    
+    def _save_object(self, obj, transaction):
+        raise NotImplementedError
+    
+    def _remove_indexes(self, obj, transaction):
+        raise NotImplementedError
+    
+    def _delete_object(self, obj, deleted, transaction):
+        raise NotImplementedError
     
     def keys(self, pattern = '*'):
         raise NotImplementedError
@@ -232,18 +277,20 @@ Called to clear a model instance.
             
     # DATASTRUCTURES
     
-    def index_keys(self, id, timeout):
+    def index_keys(self, id, timeout, transaction = None):
         return Keys(id,timeout,self._keys)
     
     def list(self, id, timeout = 0, **kwargs):
         '''Return an instance of :class:`stdnet.List`
 for a given *id*.'''
-        return self.structure_module.List(self, id, timeout = timeout, **kwargs)
+        return self.structure_module.List(self, id,
+                                          timeout = timeout, **kwargs)
     
     def hash(self, id, timeout = 0, **kwargs):
         '''Return an instance of :class:`stdnet.HashTable` structure
 for a given *id*.'''
-        return self.structure_module.HashTable(self, id, timeout = timeout, **kwargs)
+        return self.structure_module.HashTable(self, id,
+                                               timeout = timeout, **kwargs)
     
     def ts(self, id, timeout = 0, **kwargs):
         '''Return an instance of :class:`stdnet.HashTable` structure
@@ -253,11 +300,13 @@ for a given *id*.'''
     def unordered_set(self, id, timeout = 0, **kwargs):
         '''Return an instance of :class:`stdnet.Set` structure
 for a given *id*.'''
-        return self.structure_module.Set(self, id, timeout = timeout, **kwargs)
+        return self.structure_module.Set(self, id,
+                                         timeout = timeout, **kwargs)
     
     def ordered_set(self, id, timeout = 0, **kwargs):
         '''Return an instance of :class:`stdnet.OrderedSet` structure
 for a given *id*.'''
-        return self.structure_module.OrderedSet(self, id, timeout = timeout, **kwargs)
+        return self.structure_module.OrderedSet(self, id,
+                                                timeout = timeout, **kwargs)
     
 
