@@ -1,5 +1,5 @@
 '''Interfaces for supported data-structures'''
-
+import stdnet
 from stdnet.utils import iteritems
 
 __all__ = ['PipeLine',
@@ -126,19 +126,25 @@ class Structure(object):
     '''
     struct = None
     def __init__(self, cursor, id, timeout = 0,
-                 pickler = None, pipeline = None,
-                 scorefun = None, **kwargs):
-        self.cursor    = cursor
-        self.scorefun  = scorefun
-        self.pickler   = pickler if pickler is not None else cursor.pickler
-        self.id        = id
-        self.timeout   = timeout
-        self._cache    = None
-        if pipeline:
-            self._pipeline = pipeline.pipe
-            self.timeout   = pipeline.timeout
-        else:
-            self._pipeline = None
+                 pickler = None, cachepipes = None,
+                 scorefun = None, transaction = None,
+                 **kwargs):
+        self.scorefun = scorefun
+        self.transaction = transaction
+        self.pickler = pickler if pickler is not None else cursor.pickler
+        self.id = id
+        self._cache = None
+        if not transaction:
+            self.transaction = cursor.transaction(pipelined = False,
+                                                  cachepipes = cachepipes)
+        self.timeout = timeout
+        self.cursor = self.transaction.pipe
+    
+    def __get_pipeline(self):
+        pipeline = self.transaction._get_pipe(self.id,self.struct,self.timeout)
+        self.timeout = pipeline.timeout
+        return pipeline.pipe
+    pipeline = property(__get_pipeline)
     
     def __repr__(self):
         base = '%s:%s' % (self.__class__.__name__,self.id)
@@ -150,12 +156,12 @@ class Structure(object):
     def __str__(self):
         return self.__repr__()
     
-    def __get_pipeline(self):
-        if self._pipeline is not None:
-            return self._pipeline
-        else:
-            return self.cursor._get_pipe(self.id,self.struct,self.timeout).pipe
-    pipeline = property(__get_pipeline)
+    #def __get_pipeline(self):
+    #    if self._pipeline is not None:
+    #        return self._pipeline
+    #    else:
+    #        return self.cursor._get_pipe(self.id,self.struct,self.timeout).pipe
+    #pipeline = property(__get_pipeline)
         
     def size(self):
         '''Number of elements in structure'''
@@ -186,13 +192,12 @@ class Structure(object):
         return self._cache
     
     def save(self):
-        p = self.pipeline
-        if p:
-            s = self._save()
+        pipeline = self.pipeline
+        if pipeline:
+            self._save()
+            pipeline.clear()
             if self.timeout:
                 self.add_expiry()
-            p.clear()
-            return s
         else:
             return 0
         
@@ -291,7 +296,7 @@ This structure is used for in two different parts of the library.
     
     def discard(self, elem):
         '''Remove an element from a set if it is a member'''
-        return self._discard(self.pickler.dumps(elem))
+        self._discard(self.pickler.dumps(elem))
     
     def _discard(self, value):
         raise NotImplementedError
@@ -306,7 +311,6 @@ class OrderedSet(Set):
             value = self.pickler.dumps(value)
         return self._rank(value)
         
-        
     def __iter__(self):
         if not self._cache:
             self.cache = cache = []
@@ -352,8 +356,7 @@ class KeyValueStructure(Structure):
             return self._contains(value)
         else:
             return value in self._cache
-    
-    
+
     
 class HashTable(Structure):
     '''A hash-table :class:`stdnet.Structure`.
@@ -369,6 +372,7 @@ This structure is important since it is used in two different parts of the libra
 * It is also used as field (:class:`stdnet.orm.HashField`) like all other 
   class:`stdnet.Structure`.'''
     struct = HashPipe
+
     def __init__(self, *args, **kwargs):
         self.converter = kwargs.pop('converter',None) or keyconverter
         super(HashTable,self).__init__(*args, **kwargs)
@@ -389,6 +393,15 @@ This structure is important since it is used in two different parts of the libra
         self.update({key:value})
     __setitem__ = add
     
+    def addnx(self, field, value):
+        '''Set the value of a hash field only if the field
+does not exist. Otherwise it raises a :class:`stdnet.FieldValueError`
+exception.'''
+        key = self.converter.tokey(field)
+        value = self.pickler.dumps(value)
+        if not self._addnx(key,value):
+            raise stdnet.FieldValueError('Field {0} already in hash table {1}'.format(field,self))
+    
     def update(self, mapping):
         '''Add *mapping* dictionary to hashtable. Equivalent to python dictionary update method.'''
         tokey = self.converter.tokey
@@ -467,6 +480,9 @@ This structure is important since it is used in two different parts of the libra
         '''Clear the Hash table. Equivalent to ``dict.clear`` method in Python.'''
         raise NotImplementedError
     
+    def _addnx(self, field, value):
+        raise NotImplementedError
+    
     def _contains(self, value):
         raise NotImplementedError
     
@@ -487,6 +503,8 @@ This structure is important since it is used in two different parts of the libra
 
     
 class TS(HashTable):
+    '''A timeseries :class:`stdnet.Structure`.
+'''
     struct = TsPipe
         
     def update(self, mapping):
