@@ -4,35 +4,76 @@ from stdnet.exceptions import *
 from stdnet.utils import zip, UnicodeMixin
 from stdnet import dispatch
 
-from .base import StdNetType
+from .base import StdNetType, FakeModelType
 from .globals import get_model_from_hash 
 from .signals import *
 
 
-__all__ = ['StdModel',
+__all__ = ['FakeModel',
+           'StdModel',
            'StdNetType',
            'from_uuid',
            'model_to_dict']
 
 
-StdNetBase = StdNetType('StdNetBase',(UnicodeMixin,),{})
+class ModelMixin(UnicodeMixin):
+    DoesNotExist = ObjectNotFound
+    DoesNotValidate = ObjectNotValidated
+    
+    def __eq__(self, other):
+        if other.__class__ == self.__class__:
+            return self.id == other.id
+        else:
+            return False
+        
+    def __ne__(self, other):
+        return not self.__eq__(other)
+    
+    def __hash__(self):
+        try:
+            return hash(self.uuid)
+        except self.DoesNotExist as e:
+            raise TypeError(str(e))
+        
+    @property
+    def uuid(self):
+        '''Universally unique identifier for a model instance.'''
+        if not self.id:
+            raise self.DoesNotExist(\
+                    'Object not saved. Cannot obtain universally unique id')
+        return '{0}.{1}'.format(self._meta.hash,self.id)
+        
+    
+FakeModelBase = FakeModelType('FakeModelBase',(ModelMixin,),{})
+StdNetBase = StdNetType('StdNetBase',(ModelMixin,),{})
+
+
+class FakeModel(FakeModelBase):
+    id = None
+    is_base_class = True
 
 
 class StdModel(StdNetBase):
     '''A model is the single, definitive source of data
 about your data. It contains the essential fields and behaviors
 of the data you're storing. Each model class
-maps to a single :class:`stdnet.HashTable` structure via
+maps instances to :class:`stdnet.HashTable` structures via
 the :attr:`StdModel._meta` attribute.
 
 .. attribute:: _meta
 
     Instance of :class:`stdnet.orm.base.Metaclass`
+
+.. attribute:: id
+
+    Model instance id. The instance primary key.
+        
+.. attribute:: uuid
+
+    Universally unique identifier for a model instance.
     
 '''
     is_base_class = True
-    DoesNotExist = ObjectNotFound
-    DoesNotValidate = ObjectNotValidated
     
     def __init__(self, **kwargs):
         for field in self._meta.scalarfields:
@@ -43,27 +84,52 @@ the :attr:`StdModel._meta` attribute.
             setattr(self,name,value)
         setattr(self,'id',kwargs.pop('id',None))
         if kwargs:
-            raise ValueError("'%s' is an invalid keyword argument for %s" % (kwargs.keys()[0],self._meta))
-        #for field in self._meta.multifields:
-        #    setattr(self,field.attname,field.to_python(self))
+            raise ValueError("'%s' is an invalid keyword argument for %s" %\
+                              (kwargs.keys()[0],self._meta))
+        self.afterload()
+        
+    def afterload(self):
+        self._cachepipes = {}
     
-    def save(self, commit = True):
-        '''Save the instance in the remote :class:`stdnet.HashTable`
+    def save(self, transaction = None, skip_signal = False):
+        '''Save the instance.
 The model must be registered with a :class:`stdnet.backends.BackendDataServer`
-otherwise a :class:`stdnet.exceptions.ModelNotRegistered` exception will raise.'''
-        if commit:
+otherwise a :class:`stdnet.ModelNotRegistered` exception will raise.
+
+:parameter transaction: Optional transaction instance.
+                        It can be useful when saving
+                        several object together (it guarantees atomicity
+                        and it is much faster).
+                        Check the :ref:`transaction <model-transactions>`
+                        documentation for more information.
+                        
+                        Default: ``None``.
+                        
+:parameter skip_signal: When saving an instance, :ref:`signals <signal-api>`
+                        are sent just before and just after saving. If
+                        this flag is set to ``False``, those signals
+                        are not used in the function call.
+                        
+                        Default: ``False``
+                        
+The method return ``self``.
+'''
+        send_signal = not transaction and not skip_signal
+        if send_signal:
             pre_save.send(sender=self.__class__, instance = self)
         meta = self._meta
         if not meta.cursor:
             raise ModelNotRegistered("Model '{0}' is not registered with a\
  backend database. Cannot save instance.".format(meta))
-        r = meta.cursor.save_object(self, commit)
-        if commit:
+        r = meta.cursor.save_object(self, transaction)
+        if send_signal:
             post_save.send(sender=self.__class__,
                            instance = r)
         return r
     
     def is_valid(self):
+        '''Return ``True`` if the model validates.
+It check all fields agains their respective validation algorithm.'''
         return self._meta.is_valid(self)
 
     def _valattr(self, name):
@@ -82,51 +148,33 @@ otherwise a :class:`stdnet.exceptions.ModelNotRegistered` exception will raise.'
     def indices(self):
         return self._valattr('indices')
     
-    @property
-    def uuid(self):
-        '''Universally unique identifier for a model instance.'''
-        if not self.id:
-            raise self.DoesNotExist('Object not saved. Cannot obtain universally unique id')
-        return '{0}.{1}'.format(self._meta.hash,self.id)
-    
-    def __eq__(self, other):
-        if other.__class__ == self.__class__:
-            return self.id == other.id
-        else:
-            return False
-        
-    def __ne__(self, other):
-        return not self.__eq__(other)
-    
-    def __hash__(self):
-        try:
-            return hash(self.uuid)
-        except self.DoesNotExist as e:
-            raise TypeError(str(e))
-        
-    def delete(self, dlist = None):
-        '''Delete an instance from database. If the instance is not available (it does not have an id) and
-``StdNetException`` exception will raise. Return the number of model instances deleted.'''
+    def delete(self, transaction = None):
+        '''Delete an instance from database.
+If the instance is not available (it does not have an id) and
+``StdNetException`` exception will raise.
+
+:parameter transaction: Optional transaction instance as in
+                        :meth:`stdnet.orm.StdModel.save`.
+'''
         meta = self._meta
         if not self.id:
             raise StdNetException('Cannot delete object. It was never saved.')
         T = 0
         # Gather related objects to delete
         pre_delete.send(sender=self.__class__, instance = self)
-        objs = self.related_objects()
-        for obj in objs:
-            T += obj.delete(dlist)
-        res = T + meta.cursor.delete_object(self, dlist)
+        for obj in self.related_objects():
+            T += obj.delete(transaction)
+        res = T + meta.cursor.delete_object(self, transaction)
         post_delete.send(sender=self.__class__, instance = self)
         return res
     
     def related_objects(self):
-        '''Collect or related objects'''
+        '''A generator of related objects'''
         objs = []
         for rel in self._meta.related:
             rmanager = getattr(self,rel)
-            objs.extend(rmanager.all())
-        return objs
+            for obj in rmanager.all():
+                yield obj
     
     def todict(self):
         odict = {}
@@ -136,30 +184,32 @@ otherwise a :class:`stdnet.exceptions.ModelNotRegistered` exception will raise.'
             if value:
                 odict[field.name] = value
         return odict
-        
-    def afterload(self):
-        pass
     
     # UTILITY METHODS
     
     def instance_keys(self):
-        '''Utility method for returning keys associated with this instance only. The instance id
+        '''Utility method for returning keys associated with
+this instance only. The instance id
 is however available in other keys (indices and other backend containers).'''
         return self._meta.cursor.instance_keys(self)
-            
-    @classmethod
-    def commit(cls):
-        '''Shortcut to commit changes'''
-        return cls._meta.cursor.commit()
     
     @classmethod
     def flush(cls, count = None):
         '''Flush the model table and all related tables including all indexes.
-Calling flush will erase everything about the model instances in the remote server.
-If count is a dictionary, the method
+Calling flush will erase everything about the model
+instances in the remote server. If count is a dictionary, the method
 will enumerate the number of object to delete. without deleting them.'''
         return cls._meta.flush(count)
     
+    @classmethod
+    def transaction(cls):
+        '''Return a transaction instance.'''
+        c = cls._meta.cursor
+        if not c:
+            raise ModelNotRegistered("Model '{0}' is not registered with a\
+ backend database. Cannot start a transaction.".format(cls._meta))
+        return c.transaction()
+    
     # PICKLING SUPPORT
     
     def __getstate__(self):
@@ -175,7 +225,7 @@ will enumerate the number of object to delete. without deleting them.'''
             if name in data:
                 value = data[name]
             else:
-                value = None
+                value = field.value_from_data(data)
             setattr(self,name,field.to_python(value))
         self.afterload()
     
@@ -188,7 +238,8 @@ instance an exception will raise.'''
     if len(elems) == 2:
         model = get_model_from_hash(elems[0])
         if not model:
-            raise StdModel.DoesNotExist('model id "{0}" not available'.format(elems[0]))
+            raise StdModel.DoesNotExist(\
+                        'model id "{0}" not available'.format(elems[0]))
         return model.objects.get(id = elems[1])
     raise StdModel.DoesNotExist('uuid "{0}" not recognized'.format(uuid))
     
