@@ -2,7 +2,7 @@ import time
 from datetime import date, datetime
 from decimal import Decimal
 import json
-    
+
 date2timestamp = lambda dte : int(time.mktime(dte.timetuple()))
 
 
@@ -72,11 +72,3 @@ class JSONRPCEncoder(json.JSONEncoder):
         else:
             raise exceptions.JSONEncodeException("%r is not JSON serializable" % (obj,))
         
-
-class jsonPickler(object):
-    
-    def dumps(self, obj, **kwargs):
-        return json.dumps(res, cls=JSONRPCEncoder, **kwargs)
-    
-    def loads(self,sobj):
-        return json.loads(sobj)
