@@ -16,6 +16,7 @@ class SimpleModel(orm.StdModel):
     group = orm.SymbolField(required = False)
     description = orm.CharField()
     somebytes = orm.ByteField()
+    object = orm.PickleObjectField(required = False)
     
     objects = CustomManager()
     
@@ -41,6 +42,13 @@ class Base(orm.StdModel):
 class Instrument(Base):
     type = orm.SymbolField()
 
+
+class Instrument2(Base):
+    type = orm.SymbolField()
+
+    class Meta:
+        ordering = 'id'
+        
     
 class Fund(Base):
     description = orm.CharField()
@@ -104,14 +112,36 @@ class Dictionary(orm.StdModel):
     
 class SimpleList(orm.StdModel):
     names = orm.ListField()
-
-
+    
+    
 class TestDateModel(orm.StdModel):
+    person = orm.SymbolField()
     name = orm.SymbolField()
     dt = orm.DateField()
+
+
+class SportAtDate(TestDateModel):
+    
+    class Meta:
+        ordering = 'dt'
+    
+
+class SportAtDate2(TestDateModel):
+    
+    class Meta:
+        ordering = '-dt'
+    
+
+class Group(orm.StdModel):
+    name = orm.SymbolField()
     
     
-# Create the model for testing.
+class Person(orm.StdModel):
+    name = orm.SymbolField()
+    group = orm.ForeignKey(Group)
+
+    
+# A model for testing a recursive foreign key
 class Node(orm.StdModel):
     parent = orm.ForeignKey('self', required = False, related_name = 'children')
     weight = orm.FloatField()
@@ -171,7 +201,37 @@ class Statistics(orm.StdModel):
     dt = orm.DateField()
     data = orm.JSONField()
     
+    
 class Statistics2(orm.StdModel):
     dt = orm.DateField()
     data = orm.JSONField(sep = '__')
+    
+    
+class Statistics3(orm.StdModel):
+    name = orm.SymbolField()
+    data = orm.JSONField(as_string = False)
+    
+    
+    
+##############################################
+# PickleObjectField FIELD
+
+class Environment(orm.StdModel):
+    data = orm.PickleObjectField()
+    
+
+##############################################
+# Numeric Data
+
+class NumericData(orm.StdModel):
+    pv = orm.FloatField()
+    vega = orm.FloatField(default = 0.)
+    delta = orm.FloatField(default = 1.0)
+    gamma = orm.FloatField(required = False)
+    ok = orm.BooleanField()
+
+
+class DateData(orm.StdModel):
+    dt1 = orm.DateField(required = False)
+    dt2 = orm.DateTimeField(default = datetime.now)
     
\ No newline at end of file
