@@ -18,17 +18,18 @@ class TestUniqueFilter(test.TestCase):
     
     def setUp(self):
         self.orm.register(SimpleModel)
-        for n,g in zip(codes,groups):
-            SimpleModel(code = n, group = g).save(commit = False)
-        SimpleModel.commit()
+        with SimpleModel.transaction() as t:
+            for n,g in zip(codes,groups):
+                SimpleModel(code = n, group = g).save(t)
     
     def testFilterSimple(self):
         for i in range(10):
             i = randint(0,len(codes)-1)
             code = codes[i]
-            r = SimpleModel.objects.filter(code = code)
-            self.assertEqual(r.count(),1)
-            self.assertEqual(r[0].code,code)
+            qs = SimpleModel.objects.filter(code = code)
+            self.assertEqual(qs.count(),1)
+            self.assertTrue(qs.simple)
+            self.assertEqual(qs[0].code,code)
             
     def testExcludeSimple(self):
         for i in range(10):
@@ -36,4 +37,5 @@ class TestUniqueFilter(test.TestCase):
             code = codes[i]
             r = SimpleModel.objects.exclude(code = code)
             self.assertEqual(r.count(),SIZE-1)
-            self.assertFalse(code in set((o.code for o in r)))
\ No newline at end of file
+            self.assertFalse(code in set((o.code for o in r)))
+
