@@ -3,7 +3,7 @@ import threading
 import time
 from distutils.version import StrictVersion
 
-from stdnet.utils import to_string, ispy3k, iteritems, to_bytestring, range
+from stdnet.utils import to_string, ispy3k, iteritems, range
 
 if not ispy3k:
     chr = unichr
@@ -14,12 +14,7 @@ from .base import *
 to_charlist = lambda x: [x[c:c + 1] for c in range(len(x))]
 binary_set = lambda x : set(to_charlist(x))
 
-
-def get_version(info):
-    if 'redis_version' in info:
-        return info['redis_version']
-    else:
-        return info['Server']['redis_version']
+encode_dict = lambda h : dict(((k,v.encode()) for k,v in h.items()))
 
 
 class ServerCommandsTestCase(BaseTest):
@@ -47,8 +42,9 @@ class ServerCommandsTestCase(BaseTest):
         self.assert_(self.client.set('integer', 5))
         self.assert_(self.client.set('unicode_string', unicode_string))
         self.assertEquals(self.client.get('byte_string'), byte_string)
-        self.assertEquals(self.client.get('integer'), to_bytestring(integer))
-        self.assertEquals(self.client.get('unicode_string').decode('utf-8'), unicode_string)
+        self.assertEquals(int(self.client.get('integer')), integer)
+        self.assertEquals(self.client.get('unicode_string').decode('utf-8'),
+                          unicode_string)
 
     def test_getitem_and_setitem(self):
         self.client['a'] = 'bar'
@@ -141,11 +137,11 @@ class ServerCommandsTestCase(BaseTest):
 
     def test_keys(self):
         self.assertEquals(self.client.keys(), [])
-        keys = set([b'test_a', b'test_b', b'testc'])
+        keys = set(['test_a', 'test_b', 'testc'])
         for key in keys:
             self.client[key] = 1
         self.assertEquals(set(self.client.keys(pattern='test_*')),
-            keys - set([b'testc']))
+            keys - set(['testc']))
         self.assertEquals(set(self.client.keys(pattern='test*')), keys)
 
     def test_mget(self):
@@ -160,7 +156,7 @@ class ServerCommandsTestCase(BaseTest):
         d = {'a': '1', 'b': '2', 'c': '3'}
         self.assert_(self.client.mset(d))
         for k,v in iteritems(d):
-            self.assertEquals(self.client[k], to_bytestring(v))
+            self.assertEquals(self.client[k], v.encode('utf-8'))
 
     def test_msetnx(self):
         d = {'a': '1', 'b': '2', 'c': '3'}
@@ -168,7 +164,7 @@ class ServerCommandsTestCase(BaseTest):
         d2 = {'a': 'x', 'd': '4'}
         self.assert_(not self.client.msetnx(d2))
         for k,v in iteritems(d):
-            self.assertEquals(self.client[k], to_bytestring(v))
+            self.assertEquals(self.client[k], v.encode('utf-8'))
         self.assertEquals(self.client['d'], None)
 
     def test_randomkey(self):
@@ -176,7 +172,7 @@ class ServerCommandsTestCase(BaseTest):
         self.client['a'] = '1'
         self.client['b'] = '2'
         self.client['c'] = '3'
-        self.assert_(self.client.randomkey() in (b'a', b'b', b'c'))
+        self.assert_(self.client.randomkey() in ('a', 'b', 'c'))
 
     def test_rename(self):
         self.client['a'] = '1'
@@ -217,18 +213,18 @@ class ServerCommandsTestCase(BaseTest):
         self.assertEquals(self.client.substr('a', 2, -2), b'345')
 
     def test_type(self):
-        self.assertEquals(self.client.type('a'), b'none')
+        self.assertEquals(self.client.type('a'), 'none')
         self.client['a'] = '1'
-        self.assertEquals(self.client.type('a'), b'string')
+        self.assertEquals(self.client.type('a'), 'string')
         del self.client['a']
         self.client.lpush('a', '1')
-        self.assertEquals(self.client.type('a'), b'list')
+        self.assertEquals(self.client.type('a'), 'list')
         del self.client['a']
         self.client.sadd('a', '1')
-        self.assertEquals(self.client.type('a'), b'set')
+        self.assertEquals(self.client.type('a'), 'set')
         del self.client['a']
         self.client.zadd('a', '1', 1)
-        self.assertEquals(self.client.type('a'), b'zset')
+        self.assertEquals(self.client.type('a'), 'zset')
 
     # LISTS
     def make_list(self, name, l):
@@ -238,24 +234,24 @@ class ServerCommandsTestCase(BaseTest):
     def test_blpop(self):
         self.make_list('a', 'ab')
         self.make_list('b', 'cd')
-        self.assertEquals(self.client.blpop(['b', 'a'], timeout=1), (b'b', b'c'))
-        self.assertEquals(self.client.blpop(['b', 'a'], timeout=1), (b'b', b'd'))
-        self.assertEquals(self.client.blpop(['b', 'a'], timeout=1), (b'a', b'a'))
-        self.assertEquals(self.client.blpop(['b', 'a'], timeout=1), (b'a', b'b'))
-        self.assertEquals(self.client.blpop(['b', 'a'], timeout=1), None)
+        self.assertEquals(self.client.blpop(['b', 'a'],timeout=1),(b'b', b'c'))
+        self.assertEquals(self.client.blpop(['b', 'a'],timeout=1),(b'b', b'd'))
+        self.assertEquals(self.client.blpop(['b', 'a'],timeout=1),(b'a', b'a'))
+        self.assertEquals(self.client.blpop(['b', 'a'],timeout=1),(b'a', b'b'))
+        self.assertEquals(self.client.blpop(['b', 'a'],timeout=1),None)
         self.make_list('c', 'a')
         self.assertEquals(self.client.blpop('c', timeout=1), (b'c', b'a'))
 
     def test_brpop(self):
         self.make_list('a', 'ab')
         self.make_list('b', 'cd')
-        self.assertEquals(self.client.brpop(['b', 'a'], timeout=1), (b'b', b'd'))
-        self.assertEquals(self.client.brpop(['b', 'a'], timeout=1), (b'b', b'c'))
-        self.assertEquals(self.client.brpop(['b', 'a'], timeout=1), (b'a', b'b'))
-        self.assertEquals(self.client.brpop(['b', 'a'], timeout=1), (b'a', b'a'))
-        self.assertEquals(self.client.brpop(['b', 'a'], timeout=1), None)
+        self.assertEquals(self.client.brpop(['b', 'a'],timeout=1),(b'b', b'd'))
+        self.assertEquals(self.client.brpop(['b', 'a'],timeout=1),(b'b', b'c'))
+        self.assertEquals(self.client.brpop(['b', 'a'],timeout=1),(b'a', b'b'))
+        self.assertEquals(self.client.brpop(['b', 'a'],timeout=1),(b'a', b'a'))
+        self.assertEquals(self.client.brpop(['b', 'a'],timeout=1),None)
         self.make_list('c', 'a')
-        self.assertEquals(self.client.brpop('c', timeout=1), (b'c', b'a'))
+        self.assertEquals(self.client.brpop('c', timeout=1),(b'c', b'a'))
 
     def test_lindex(self):
         # no key
@@ -480,7 +476,7 @@ class ServerCommandsTestCase(BaseTest):
         # test for that
         # real logic
         self.assertEquals(self.client.sdiffstore('c', ['a', 'b']), 2)
-        self.assertEquals(self.client.smembers('c'), set([b'a1', b'a3']))
+        self.assertEquals(self.client.smembers('c'), set((b'a1', b'a3')))
 
     def test_sinter(self):
         # some key is not a set
@@ -585,8 +581,8 @@ class ServerCommandsTestCase(BaseTest):
         del self.client['a']
         # real logic
         self.make_set('a', 'abc')
-        self.assertEquals(self.client.srem('a', b'd'), False)
-        self.assertEquals(self.client.srem('a', b'b'), True)
+        self.assertEquals(self.client.srem('a', 'd'), False)
+        self.assertEquals(self.client.srem('a', 'b'), True)
         bs = binary_set(b'ac')
         self.assertEquals(self.client.smembers('a'), bs)
 
@@ -702,12 +698,14 @@ class ServerCommandsTestCase(BaseTest):
         self.make_zset('a', {'a1': 1, 'a2': 2, 'a3': 3, 'a4': 4, 'a5': 5})
         self.assertEquals(list(self.client.zrangebyscore('a', 2, 4)),
             [b'a2', b'a3', b'a4'])
-        self.assertEquals(list(self.client.zrangebyscore('a', 2, 4, start=1, num=2)),
-            [b'a3', b'a4'])
-        self.assertEquals(list(self.client.zrangebyscore('a', 2, 4, withscores=True)),
+        self.assertEquals(list(self.client.zrangebyscore(
+                            'a', 2, 4, start=1, num=2)),[b'a3', b'a4'])
+        self.assertEquals(list(self.client.zrangebyscore(
+            'a', 2, 4, withscores=True)),
             [(b'a2', 2.0), (b'a3', 3.0), (b'a4', 4.0)])
         # a non existant key should return empty list
-        self.assertEquals(self.client.zrangebyscore('b', 0, 1, withscores=True), [])
+        self.assertEquals(self.client.zrangebyscore(
+                                'b', 0, 1, withscores=True), [])
 
     def test_zrank(self):
         # key is not a zset
@@ -759,9 +757,11 @@ class ServerCommandsTestCase(BaseTest):
         self.make_zset('a', {'a1': 1, 'a2': 2, 'a3': 3})
         self.assertEquals(self.client.zrevrange('a', 0, 1), [b'a3', b'a2'])
         self.assertEquals(self.client.zrevrange('a', 1, 2), [b'a2', b'a1'])
-        self.assertEquals(list(self.client.zrevrange('a', 0, 1, withscores=True)),
+        self.assertEquals(list(self.client.zrevrange(
+            'a', 0, 1, withscores=True)),
             [(b'a3', 3.0), (b'a2', 2.0)])
-        self.assertEquals(list(self.client.zrevrange('a', 1, 2, withscores=True)),
+        self.assertEquals(list(self.client.zrevrange(
+            'a', 1, 2, withscores=True)),
             [(b'a2', 2.0), (b'a1', 1.0)])
         # a non existant key should return empty list
         self.assertEquals(self.client.zrange('b', 0, 1, withscores=True), [])
@@ -853,16 +853,16 @@ class ServerCommandsTestCase(BaseTest):
         self.assertEqual(self.client.hget('a', 'a1'), b'1')
 
     def test_hmset(self):
-        d = {'a': '1', 'b': '2', 'c': '3'}
-        db = dict(((to_bytestring(k),to_bytestring(v)) for k,v in d.items()))
-        self.assert_(self.client.hmset('foo', d))
-        self.assertEqual(self.client.hgetall('foo'), db)
+        db = {'a': '1', 'b': '2', 'c': '3'}
+        self.assert_(self.client.hmset('foo', db))
+        self.assertEqual(dict(self.client.hgetall('foo')), encode_dict(db))
         self.assertRaises(ResponseError, self.client.hmset, 'foo', {})
 
     def test_hmget(self):
         d = {'a': 1, 'b': 2, 'c': 3}
         self.assert_(self.client.hmset('foo', d))
-        self.assertEqual(self.client.hmget('foo', ['a', 'b', 'c']), [b'1', b'2', b'3'])
+        self.assertEqual(self.client.hmget('foo',
+                                     ['a', 'b', 'c']), [b'1', b'2', b'3'])
         self.assertEqual(self.client.hmget('foo', ['a', 'c']), [b'1', b'3'])
 
     def test_hmget_empty(self):
@@ -904,12 +904,12 @@ class ServerCommandsTestCase(BaseTest):
         self.assertRaises(ResponseError, self.client.hgetall, 'a')
         del self.client['a']
         # no key
-        self.assertEquals(self.client.hgetall('a'), {})
+        self.assertEquals(self.client.hgetall('a'), ())
         # real logic
         h = {'a1': '1', 'a2': '2', 'a3': '3'}
-        db = dict(((to_bytestring(k),to_bytestring(v)) for k,v in h.items()))
+        db = encode_dict(h)
         self.make_hash('a', h)
-        remote_hash = self.client.hgetall('a')
+        remote_hash = dict(self.client.hgetall('a'))
         self.assertEquals(db, remote_hash)
 
     def test_hincrby(self):
@@ -941,7 +941,7 @@ class ServerCommandsTestCase(BaseTest):
         # real logic
         h = {'a1': '1', 'a2': '2', 'a3': '3'}
         self.make_hash('a', h)
-        keys = sorted((to_bytestring(k) for k in h.keys()))
+        keys = sorted((k for k in h.keys()))
         remote_keys = sorted(self.client.hkeys('a'))
         self.assertEquals(keys, remote_keys)
 
@@ -968,7 +968,7 @@ class ServerCommandsTestCase(BaseTest):
         # real logic
         h = {'a1': '1', 'a2': '2', 'a3': '3'}
         self.make_hash('a', h)
-        vals = sorted((to_bytestring(v) for v in h.values()))
+        vals = sorted((v.encode('utf-8') for v in h.values()))
         remote_vals = sorted(self.client.hvals('a'))
         self.assertEquals(vals, remote_vals)
 
@@ -1082,14 +1082,13 @@ class ServerCommandsTestCase(BaseTest):
         for msg_type, channel, _ in messages:
             sent_types.setdefault(msg_type, 0)
             sent_types[msg_type] += 1
-            if msg_type == b'message':
+            if msg_type == 'message':
                 sent_channels.setdefault(channel, 0)
                 sent_channels[channel] += 1
         for channel in channels:
-            channel = to_bytestring(channel)
             self.assertEquals(sent_channels[channel], messages_per_channel)
-        self.assertEquals(sent_types[b'subscribe'], len(channels))
-        self.assertEquals(sent_types[b'message'],
+        self.assertEquals(sent_types['subscribe'], len(channels))
+        self.assertEquals(sent_types['message'],
             len(channels) * messages_per_channel)
 
     ## BINARY SAFE
@@ -1098,13 +1097,14 @@ class ServerCommandsTestCase(BaseTest):
         self.assertTrue(self.client.set(' foo bar ', '123'))
         self.assertEqual(self.client.get(' foo bar '), b'123')
 
-        self.assertTrue(self.client.set(' foo\r\nbar\r\n ', b'456'))
+        self.assertTrue(self.client.set(' foo\r\nbar\r\n ', '456'))
         self.assertEqual(self.client.get(' foo\r\nbar\r\n '), b'456')
 
         self.assertTrue(self.client.set(' \r\n\t\x07\x13 ', '789'))
         self.assertEqual(self.client.get(' \r\n\t\x07\x13 '), b'789')
 
-        self.assertEqual(sorted(self.client.keys('*')), [b' \r\n\t\x07\x13 ', b' foo\r\nbar\r\n ', b' foo bar '])
+        self.assertEqual(sorted(self.client.keys('*')),
+                 [' \r\n\t\x07\x13 ', ' foo\r\nbar\r\n ', ' foo bar '])
 
         self.assertTrue(self.client.delete(' foo bar '))
         self.assertTrue(self.client.delete(' foo\r\nbar\r\n '))
@@ -1121,10 +1121,10 @@ class ServerCommandsTestCase(BaseTest):
                 self.assertTrue(self.client.rpush(key, c))
 
         # check that KEYS returns all the keys as they are
-        bk = sorted((to_bytestring(k) for k in mapping.keys()))
+        bk = sorted((k for k in mapping.keys()))
         self.assertEqual(sorted(self.client.keys('*')), bk)
 
         # check that it is possible to get list content by key name
         for key in mapping.keys():
-            bl = [to_bytestring(v) for v in mapping[key]]
+            bl = [v.encode() for v in mapping[key]]
             self.assertEqual(self.client.lrange(key, 0, -1), bl)
