@@ -6,31 +6,36 @@ from decimal import Decimal
 
 import stdnet
 from stdnet import test
-from stdnet.utils import populate, zip, is_string, to_string
+from stdnet.utils import populate, zip, is_string, to_string, unichr, ispy3k
 from stdnet.exceptions import FieldError
 
 from examples.models import TestDateModel, Statistics, Statistics2,\
-                            Page, SimpleModel
+                            Page, SimpleModel, Environment, NumericData,\
+                            DateData, Statistics3
 
 NUM_DATES = 100
 names = populate('string',NUM_DATES, min_len = 5, max_len = 20)
 dates = populate('date', NUM_DATES, start=date(2010,5,1), end=date(2010,6,1))
 
 
-class TestAtomFields(test.TestCase):
+class TestScalarBase(test.TestCase):
+    model = None
     
-    def setUp(self):
-        self.orm.register(TestDateModel)
-        self.meta = TestDateModel._meta
+    def register(self):
+        self.orm.register(self.model)
+        self.meta = self.model._meta
     
     def unregister(self):
-        self.orm.unregister(TestDateModel)
+        self.orm.unregister(self.model)
+
+
+class TestAtomFields(TestScalarBase):
+    model = TestDateModel
         
     def create(self):
-        for na,dt in zip(names,dates):
-            m = TestDateModel(name = na, dt = dt)
-            m.save(False)
-        TestDateModel.commit()
+        with TestDateModel.transaction() as t:
+            for na,dt in zip(names,dates):
+                TestDateModel(person = na, name = na, dt = dt).save(t)
             
     def testFilter(self):
         self.create()
@@ -60,20 +65,53 @@ class TestAtomFields(test.TestCase):
         all = TestDateModel.objects.all()
         self.assertEqual(all.count(),0)
         
-        # The only key remianed is the ids key for the AutoField
-        keys = self.meta.cursor.keys()
+        # The only key remaining is the ids key for the AutoField
+        keys = self.cleankeys(self.meta)
         self.assertEqual(len(keys),1)
         self.assertEqual(keys[0],self.meta.autoid())
         
 
-class TestIntegerField(test.TestCase):
+class TestCharFields(TestScalarBase):
+    model = SimpleModel
     
-    def setUp(self):
-        self.orm.register(Page)
-    
-    def unregister(self):
-        self.orm.unregister(Page)
+    def testUnicode(self):
+        unicode_string = unichr(500) + to_string('ciao') + unichr(300)
+        m = self.model(code = unicode_string).save()
+        code = m.todict()['code']
+        if ispy3k:
+            self.assertEqual(str(m),unicode_string)
+        else:
+            self.assertEqual(str(m),code)
+        self.assertTrue(isinstance(code,bytes))
+        self.assertEqual(code.decode('utf-8'),unicode_string)
+        m = self.model.objects.get(id = m.id)
+        self.assertEqual(m.code,unicode_string)
         
+    
+class TestNumericData(TestScalarBase):
+    model = NumericData
+        
+    def testDefaultValue(self):
+        d = NumericData(pv = 1.).save()
+        self.assertAlmostEqual(d.pv,1.)
+        self.assertAlmostEqual(d.vega,0.)
+        self.assertAlmostEqual(d.delta,1.)
+        self.assertEqual(d.gamma,None)
+        
+    def testDefaultValue2(self):
+        d = NumericData(pv = 0., delta = 0.).save()
+        self.assertAlmostEqual(d.pv,0.)
+        self.assertAlmostEqual(d.vega,0.)
+        self.assertAlmostEqual(d.delta,0.)
+        self.assertEqual(d.gamma,None)
+        
+    def testFieldError(self):
+        self.assertRaises(stdnet.FieldError,NumericData().save)
+                
+        
+class TestIntegerField(TestScalarBase):
+    model = Page
+            
     def testDefaultValue(self):
         p = Page()
         self.assertEqual(p.in_navigation,1)
@@ -98,6 +136,38 @@ class TestIntegerField(test.TestCase):
         self.assertEqual(p.in_navigation,0)
                
 
+class TestDateData(TestScalarBase):
+    model = DateData
+        
+    def testDateindateTime(self):
+        v = DateData(dt2 = date.today()).save()
+        v = DateData.objects.get(id = v.id)
+        self.assertEqual(v.dt1,None)
+        self.assertEqual(v.dt2.date(),date.today())
+        
+
+class TestBoolField(TestScalarBase):
+    model = NumericData
+    
+    def testMeta(self):
+        self.assertEqual(len(self.meta.indices),1)
+        index = self.meta.indices[0]
+        self.assertEqual(index.type,'bool')
+        self.assertEqual(index.scorefun(True),1)
+        self.assertEqual(index.scorefun(False),0)
+        
+    def testBoolValue(self):
+        d = self.model(pv = 1.).save()
+        d = self.model.objects.get(id = d.id)
+        self.assertEqual(d.ok,False)
+        d.ok = 'jasxbhjaxsbjxsb'
+        self.assertRaises(ValueError,d.save)
+        d.ok = True
+        d.save()
+        d = self.model.objects.get(id = d.id)
+        self.assertEqual(d.ok,True)
+        
+    
 class TestJsonField(test.TestCase):
     
     def setUp(self):
@@ -151,7 +221,7 @@ class TestJsonField(test.TestCase):
 
 class TestJsonFieldSep(test.TestCase):
     
-    def setUp(self):
+    def register(self):
         self.orm.register(Statistics2)
     
     def unregister(self):
@@ -165,11 +235,12 @@ class TestJsonFieldSep(test.TestCase):
         mean = Decimal('56.4')
         started = date(2010,1,1)
         timestamp = datetime.now()
-        a = Statistics2(dt = date.today(), data = {'stat__mean': mean,
-                                                   'stat__std': 5.78,
-                                                   'stat__secondary__ar': 0.1,
-                                                   'date__started': started,
-                                                   'date__timestamp':timestamp}).save()
+        a = Statistics2(dt = date.today(),
+                        data = {'stat__mean': mean,
+                                'stat__std': 5.78,
+                                'stat__secondary__ar': 0.1,
+                                'date__started': started,
+                                'date__timestamp':timestamp}).save()
         a = Statistics2.objects.get(id = a.id)
         self.assertEqual(len(a.data),2)
         stat = a.data['stat']
@@ -183,6 +254,41 @@ class TestJsonFieldSep(test.TestCase):
         self.assertEqual(dt['timestamp'],timestamp)
         
 
+class TestJsonFieldAsData(TestScalarBase):
+    '''Test a model with a JSONField which expand as instance fields.'''
+    model = Statistics3
+    
+    def make(self):
+        data = {'mean': 1.0,
+                'std': 5.78,
+                'pv': 3.2,
+                'name': 'bla',
+                'dt': date.today()}
+        return self.model(name = 'bla', data = data)
+        
+    def testMeta(self):
+        field = self.meta.dfields['data']
+        self.assertFalse(field.as_string)
+        
+    def testMake(self):
+        m = self.make()
+        self.assertTrue(m.is_valid())
+        data = m.cleaned_data
+        self.assertEqual(len(data),6)
+        self.assertEqual(float(data['data__mean']),1.0)
+        self.assertEqual(float(data['data__std']),5.78)
+        self.assertEqual(float(data['data__pv']),3.2)
+        
+    def testGet(self):
+        m = self.make().save()
+        m = self.model.objects.get(id = m.id)
+        self.assertEqual(m.data['mean'],1.0)
+        self.assertEqual(m.data['std'],5.78)
+        self.assertEqual(m.data['pv'],3.2)
+        self.assertEqual(m.data['dt'],date.today())
+        self.assertEqual(m.data['name'],'bla')
+        
+    
 class TestByteField(test.TestCase):
     
     def setUp(self):
@@ -212,6 +318,29 @@ class TestByteField(test.TestCase):
         self.assertFalse(is_string(v.somebytes))
         self.assertEqual(v.somebytes,b)
 
+
+class TestPickleObjectField(test.TestCase):
+    
+    def setUp(self):
+        self.orm.register(Environment)
+    
+    def unregister(self):
+        self.orm.unregister(Environment)
+        
+    def testOkObject(self):
+        v = Environment(data = ['ciao','pippo']).save()
+        self.assertEqual(v.data, ['ciao','pippo'])
+        v = Environment.objects.get(id = v.id)
+        self.assertEqual(v.data, ['ciao','pippo'])
+        
+    def testRecursive(self):
+        '''Silly test to test both pickle field and pickable instace'''
+        v = Environment(data = ('ciao','pippo', 4, {})).save()
+        v2 = Environment(data = v).save()
+        v3 = Environment.objects.get(id = v2.id)
+        self.assertEqual(v3.data, v)
+    
+
 class TestErrorAtomFields(test.TestCase):
     
     def testNotRegistered(self):
